@@ -7,8 +7,8 @@ from typing import (
     Dict,
 )
 
-from .helper import Offset2ID
-from .... import Document
+from docarray.array.storage.base.helper import Offset2ID
+from docarray import Document, DocumentArray
 
 
 class BaseGetSetDelMixin(ABC):
@@ -114,6 +114,7 @@ class BaseGetSetDelMixin(ABC):
         self._del_docs(ids)
 
     def _del_all_docs(self):
+        self._clear_subindices()
         self._clear_storage()
         self._offset2ids = Offset2ID()
 
@@ -126,6 +127,12 @@ class BaseGetSetDelMixin(ABC):
         for _id in ids:
             self._del_doc_by_id(_id)
 
+    def _update_subindices_del(self, ids):
+        if getattr(self, '_subindices', None):
+            for selector, da in self._subindices.items():
+                ids_subindex = DocumentArray(self[ids])[selector, 'id']
+                da._del_docs_by_ids(ids_subindex)
+
     def _del_docs(self, ids):
         self._del_docs_by_ids(ids)
         self._offset2ids.delete_by_ids(ids)
@@ -138,6 +145,11 @@ class BaseGetSetDelMixin(ABC):
         for doc in self:
             self._del_doc_by_id(doc.id)
 
+    def _clear_subindices(self):
+        if getattr(self, '_subindices', None):
+            for selector, da in self._subindices.items():
+                da._del_all_docs()
+
     # Setitem API
 
     def _set_doc_by_offset(self, offset: int, value: 'Document'):
@@ -161,6 +173,16 @@ class BaseGetSetDelMixin(ABC):
         for _id, doc in zip(ids, docs):
             self._set_doc_by_id(_id, doc)
 
+    def _update_subindices_set(self, ids, docs):
+        if getattr(self, '_subindices', None):
+            for selector, da in self._subindices.items():
+                old_ids = DocumentArray(self[ids])[
+                    selector, 'id'
+                ]  # hack to get the Document['@c'] without having to do Document.chunks
+                with da:
+                    del da[old_ids]
+                    da.extend(DocumentArray(docs)[selector])  # same hack here
+
     def _set_docs(self, ids, docs: Iterable['Document']):
         docs = list(docs)
         mismatch_ids = {_id: doc.id for _id, doc in zip(ids, docs) if _id != doc.id}
@@ -235,6 +257,10 @@ class BaseGetSetDelMixin(ABC):
         :param attr: the attribute of document to update
         :param value: the value doc's attr will be updated to
         """
+        if attr == 'id' and value is None:
+            raise ValueError(
+                'setting the ID of a Document stored in a DocumentArray to None is not allowed'
+            )
         _id = self._offset2ids.get_id(offset)
         d = self._get_doc_by_id(_id)
         if hasattr(d, attr):
